@@ -1,21 +1,21 @@
 /*
- * This file is part of oFono - Open Source Telephony
  *
- * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
+ *  oFono - Open Source Telephony
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * version 2 as published by the Free Software Foundation.
+ *  Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
- * 02110-1301 USA
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -45,7 +45,7 @@
 
 struct sim_data {
 	GIsiClient *client;
-	bool registered;
+	gboolean registered;
 };
 
 struct file_info {
@@ -54,6 +54,7 @@ struct file_info {
 	int structure;
 	int record_length;
 	unsigned char access[3];
+	unsigned char file_status;
 };
 
 /* Returns file info */
@@ -66,7 +67,7 @@ static gboolean fake_file_info(gpointer user)
 	DBG("Returning static file_info for %04x", fi->fileid);
 	CALLBACK_WITH_SUCCESS(cb,
 				fi->length, fi->structure, fi->record_length,
-				fi->access, cbd->data);
+				fi->access, fi->file_status, cbd->data);
 	g_free(cbd);
 	return FALSE;
 }
@@ -76,8 +77,8 @@ static void isi_read_file_info(struct ofono_sim *sim, int fileid,
 {
 	int i;
 	static struct file_info const info[] = {
-		{ SIM_EFSPN_FILEID, 17, 0, 0, { 0x0f, 0xff, 0xff } },
-		{ SIM_EF_ICCID_FILEID, 10, 0, 0, { 0x0f, 0xff, 0xff } },
+		{ SIM_EFSPN_FILEID, 17, 0, 0, { 0x0f, 0xff, 0xff }, 1 },
+		{ SIM_EF_ICCID_FILEID, 10, 0, 0, { 0x0f, 0xff, 0xff }, 1 },
 	};
 	int N = sizeof(info) / sizeof(info[0]);
 	struct isi_cb_data *cbd;
@@ -91,11 +92,12 @@ static void isi_read_file_info(struct ofono_sim *sim, int fileid,
 	}
 
 	DBG("Not implemented (fileid = %04x)", fileid);
-	CALLBACK_WITH_FAILURE(cb, -1, -1, -1, NULL, data);
+	CALLBACK_WITH_FAILURE(cb, -1, -1, -1, NULL, 0, data);
 }
 
-static bool spn_resp_cb(GIsiClient *client, const void *restrict data,
-			size_t len, uint16_t object, void *opaque)
+static gboolean spn_resp_cb(GIsiClient *client,
+				const void *restrict data, size_t len,
+				uint16_t object, void *opaque)
 {
 	const unsigned char *msg = data;
 	struct isi_cb_data *cbd = opaque;
@@ -111,7 +113,7 @@ static bool spn_resp_cb(GIsiClient *client, const void *restrict data,
 
 	if (len < 39 || msg[0] != SIM_SERV_PROV_NAME_RESP
 		|| msg[1] != SIM_ST_READ_SERV_PROV_NAME)
-		return false;
+		return FALSE;
 
 	if (msg[2] != SIM_SERV_OK) {
 		DBG("Request failed: %s (0x%02X)",
@@ -141,10 +143,10 @@ done:
 		CALLBACK_WITH_FAILURE(cb, NULL, 0, cbd->data);
 
 	g_free(cbd);
-	return true;
+	return TRUE;
 }
 
-static bool isi_read_spn(struct ofono_sim *sim, struct isi_cb_data *cbd)
+static gboolean isi_read_spn(struct ofono_sim *sim, struct isi_cb_data *cbd)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
 
@@ -154,12 +156,16 @@ static bool isi_read_spn(struct ofono_sim *sim, struct isi_cb_data *cbd)
 		0
 	};
 
+	if (!sd)
+		return FALSE;
+
 	return g_isi_request_make(sd->client, msg, sizeof(msg),
 					SIM_TIMEOUT, spn_resp_cb, cbd) != NULL;
 }
 
-static bool read_iccid_resp_cb(GIsiClient *client, const void *restrict data,
-				size_t len, uint16_t object, void *user)
+static gboolean read_iccid_resp_cb(GIsiClient *client,
+					const void *restrict data, size_t len,
+					uint16_t object, void *user)
 {
 	struct isi_cb_data *cbd = user;
 	ofono_sim_read_cb_t cb = cbd->cb;
@@ -172,7 +178,7 @@ static bool read_iccid_resp_cb(GIsiClient *client, const void *restrict data,
 	}
 
 	if (len < 3 || msg[0] != SIM_READ_FIELD_RESP || msg[1] != ICC)
-		return false;
+		return FALSE;
 
 	if (msg[2] == SIM_SERV_OK && len >= 13)
 		iccid = msg + 3;
@@ -186,14 +192,17 @@ done:
 		CALLBACK_WITH_FAILURE(cb, NULL, 0, cbd->data);
 
 	g_free(cbd);
-	return true;
+	return TRUE;
 }
 
-static bool isi_read_iccid(struct ofono_sim *sim, struct isi_cb_data *cbd)
+static gboolean isi_read_iccid(struct ofono_sim *sim, struct isi_cb_data *cbd)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
 	const unsigned char req[] = { SIM_READ_FIELD_REQ, ICC };
 
+	if (!sd)
+		return FALSE;
+
 	return g_isi_request_make(sd->client, req, sizeof(req), SIM_TIMEOUT,
 					read_iccid_resp_cb, cbd) != NULL;
 }
@@ -269,8 +278,9 @@ static void isi_write_file_cyclic(struct ofono_sim *sim, int fileid,
 	CALLBACK_WITH_FAILURE(cb, data);
 }
 
-static bool imsi_resp_cb(GIsiClient *client, const void *restrict data,
-				size_t len, uint16_t object, void *opaque)
+static gboolean imsi_resp_cb(GIsiClient *client,
+				const void *restrict data, size_t len,
+				uint16_t object, void *opaque)
 {
 	const unsigned char *msg = data;
 	struct isi_cb_data *cbd = opaque;
@@ -320,7 +330,7 @@ error:
 
 out:
 	g_free(cbd);
-	return true;
+	return TRUE;
 }
 
 static void isi_read_imsi(struct ofono_sim *sim,
@@ -333,11 +343,10 @@ static void isi_read_imsi(struct ofono_sim *sim,
 		READ_IMSI
 	};
 
-	if (!cbd)
+	if (!cbd || !sd)
 		goto error;
 
-	if (g_isi_request_make(sd->client, msg, sizeof(msg),
-				SIM_TIMEOUT,
+	if (g_isi_request_make(sd->client, msg, sizeof(msg), SIM_TIMEOUT,
 				imsi_resp_cb, cbd))
 		return;
 
@@ -351,30 +360,31 @@ static void isi_sim_register(struct ofono_sim *sim)
 	struct sim_data *sd = ofono_sim_get_data(sim);
 
 	if (!sd->registered) {
-		sd->registered = true;
+		sd->registered = TRUE;
 		ofono_sim_register(sim);
 		ofono_sim_inserted_notify(sim, TRUE);
 	}
 }
 
-static bool read_hplmn_resp_cb(GIsiClient *client, const void *restrict data,
-				size_t len, uint16_t object, void *opaque)
+static gboolean read_hplmn_resp_cb(GIsiClient *client,
+					const void *restrict data, size_t len,
+					uint16_t object, void *opaque)
 {
 	const unsigned char *msg = data;
 	struct ofono_sim *sim = opaque;
 
 	if (!msg) {
 		DBG("ISI client error: %d", g_isi_client_error(client));
-		return true;
+		return TRUE;
 	}
 
 	if (len < 3 || msg[0] != SIM_NETWORK_INFO_RESP || msg[1] != READ_HPLMN)
-		return false;
+		return FALSE;
 
 	if (msg[2] != SIM_SERV_NOTREADY)
 		isi_sim_register(sim);
 
-	return true;
+	return TRUE;
 }
 
 
@@ -391,8 +401,9 @@ static void isi_read_hplmn(struct ofono_sim *sim)
 				read_hplmn_resp_cb, sim);
 }
 
-static void sim_ind_cb(GIsiClient *client, const void *restrict data,
-			size_t len, uint16_t object, void *opaque)
+static void sim_ind_cb(GIsiClient *client,
+			const void *restrict data, size_t len,
+			uint16_t object, void *opaque)
 {
 	struct ofono_sim *sim = opaque;
 	struct sim_data *sd = ofono_sim_get_data(sim);
@@ -411,7 +422,7 @@ static void sim_ind_cb(GIsiClient *client, const void *restrict data,
 	}
 }
 
-static void sim_reachable_cb(GIsiClient *client, bool alive,
+static void sim_reachable_cb(GIsiClient *client, gboolean alive,
 				uint16_t object, void *opaque)
 {
 	struct ofono_sim *sim = opaque;
