@@ -1,21 +1,21 @@
 /*
- * oFono - GSM Telephony Stack for Linux
  *
- * Copyright (C) 2008-2010 Intel Corporation.  All rights reserved.
+ *  oFono - Open Source Telephony
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -785,7 +785,7 @@ static void test_assembly()
 	if (g_test_verbose())
 		g_printf("Text:\n%s\n", utf8);
 
-	l = sms_text_prepare(utf8, ref, TRUE, NULL, FALSE);
+	l = sms_text_prepare("555", utf8, ref, TRUE, FALSE);
 	g_assert(l);
 	g_assert(g_slist_length(l) == 3);
 
@@ -815,7 +815,8 @@ static void test_prepare_7bit()
 	int encoded_tpdu_len;
 	char *encoded_pdu;
 
-	r = sms_text_prepare(test_no_fragmentation_7bit, 0, FALSE, NULL, FALSE);
+	r = sms_text_prepare("555", test_no_fragmentation_7bit, 0,
+				FALSE, FALSE);
 
 	g_assert(r != NULL);
 
@@ -897,7 +898,7 @@ static void test_prepare_concat(gconstpointer data)
 	if (g_test_verbose())
 		g_print("strlen: %zd\n", strlen(test->str));
 
-	r = sms_text_prepare(test->str, 0, TRUE, NULL, FALSE);
+	r = sms_text_prepare("+15554449999", test->str, 0, TRUE, FALSE);
 	g_assert(r);
 	g_assert(g_slist_length(r) == test->segments);
 
@@ -906,7 +907,6 @@ static void test_prepare_concat(gconstpointer data)
 
 		sms = l->data;
 
-		sms_address_from_string(&sms->submit.daddr, "+15554449999");
 		sms_encode(sms, &pdu_len, &tpdu_len, pdu);
 		g_assert(pdu_len == (tpdu_len + 1));
 
@@ -973,7 +973,7 @@ static void test_limit(gunichar uni, int target_size, gboolean use_16bit)
 
 	utf8[i] = '\0';
 
-	l = sms_text_prepare(utf8, 0, use_16bit, NULL, FALSE);
+	l = sms_text_prepare("555", utf8, 0, use_16bit, FALSE);
 
 	g_assert(l);
 	g_assert(g_slist_length(l) == 255);
@@ -986,7 +986,7 @@ static void test_limit(gunichar uni, int target_size, gboolean use_16bit)
 	memcpy(utf8 + i, utf8_char, stride);
 	utf8[i+stride] = '\0';
 
-	l = sms_text_prepare(utf8, 0, use_16bit, NULL, FALSE);
+	l = sms_text_prepare("555", utf8, 0, use_16bit, FALSE);
 
 	g_assert(l == NULL);
 	g_free(utf8);
